package agency;

import javax.naming.*;
import javax.jms.*;

public class MessageSender {

    private Connection connection;
    private Session session;
    private MessageProducer producer;

    public MessageSender(String jndiFactory, String jndiQueue) throws JMSException, NamingException {
        Context context = new InitialContext();
        ConnectionFactory connectionFactory = (ConnectionFactory)context.lookup(jndiFactory);
        connection = connectionFactory.createConnection();
        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        Destination destination = (Destination)context.lookup(jndiQueue);
        producer = session.createProducer(destination);
    }

    public void sendApplicant(String applicant, boolean newApplicant) throws JMSException {
        TextMessage message = session.createTextMessage();
        message.setBooleanProperty ("NewApplicant", newApplicant);
        message.setText(applicant);
        producer.send(message);
    }

    public void sendJob(String ref, String customer, boolean newJob) throws JMSException {
        TextMessage message = session.createTextMessage();
        message.setBooleanProperty ("NewJob", newJob);
        message.setText(ref+"+"+customer);
        producer.send(message);
    }

    public void close() throws JMSException {
        connection.close();
    }
}

